"""
Testing suite for aat_wrest
"""

import argparse
import logging
import sys
from typing import List, Tuple
from unittest.mock import MagicMock, patch

import pendulum as pendulum
import pytest
from aat_wrest.metadata_wrester import WrestWorkflowMetadata

logger = logging.getLogger("test_aat_wrest")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


args = argparse.Namespace(stdcals="uid://evla/calibration/143a3ef9-21fb-46d8-8312-91de54b0cc49")
result = (
    '{"sdmId": "17B-197.sb34663512.eb34806505.58108.78427738426", "projectCode": "17B-197", '
    '"title": "Radio-Active CoWS: Extended FIRST Sources in MaDCoWS Clusters", '
    '"startTime": 58108.7842916667, "observer": "Anthony Gonzalez", "telescope": "EVLA",'
    ' "created_at": ' + str(pendulum.now().in_timezone("UTC").format("YYYY-MM-DDTHH:mm:ss")) + "}"
)


def mock_wrester(args: argparse.Namespace) -> WrestWorkflowMetadata:
    with patch("psycopg2.connect") as mock_connect:
        return WrestWorkflowMetadata(connection=mock_connect, spl=args.stdcals)


@pytest.fixture
def fake_obs_metadata() -> List[Tuple[str]]:
    return [
        (
            "uid://evla/execblock/8fbfb54b-d141-42fe-b079-609339a69cba",
            "Ka",
            "D",
            "1234567",
            "123598898",
        )
    ]


wrester = mock_wrester(args)


class TestAatWrest:
    def test_init(self, fake_obs_metadata: List[Tuple[str]]):
        fileset_id = "17B-197.sb34663512.eb34806505.58108.78427738426"
        with patch("psycopg2.connect") as mock_connect:
            mock_connect.cursor.return_value.__enter__.return_value.fetchall.return_value = (
                fake_obs_metadata
            )
            wrester_no_spl = WrestWorkflowMetadata(
                connection=mock_connect, spl=None, fileset_id=fileset_id
            )
            assert wrester_no_spl.spl == "uid://evla/execblock/8fbfb54b-d141-42fe-b079-609339a69cba"

    @patch("json.dumps", MagicMock(return_value=result))
    def test_wrest_standard_cal_info(self):
        wrester.conn.cursor.return_value.fetchall.return_value = [
            "17B-197.sb34663512.eb34806505.58108.78427738426",
            "17B-197",
            "Radio-Active CoWS: Extended FIRST Sources in MaDCoWS Clusters",
            58108.7842916667,
            "Anthony Gonzalez",
            "EVLA",
            str(pendulum.now().in_timezone("UTC").format("YYYY-MM-DDTHH:mm:ss")),
        ]
        assert args.stdcals == "uid://evla/calibration/143a3ef9-21fb-46d8-8312-91de54b0cc49"
        value = wrester.wrest_standard_cal_info()
        assert (
            value
            == '{"sdmId": "17B-197.sb34663512.eb34806505.58108.78427738426", "projectCode": "17B-197", '
            '"title": "Radio-Active CoWS: Extended FIRST Sources in MaDCoWS Clusters", '
            '"startTime": 58108.7842916667, "observer": "Anthony Gonzalez", "telescope": "EVLA",'
            ' "created_at": '
            + str(pendulum.now().in_timezone("UTC").format("YYYY-MM-DDTHH:mm:ss"))
            + "}"
        )

    def test_wrest_obs_metadata_from_fileset_id(self, fake_obs_metadata: List[Tuple[str]]):
        wrester.conn.cursor.return_value.__enter__.return_value.fetchall.return_value = (
            fake_obs_metadata
        )
        metadata = wrester.wrest_obs_metadata_from_fileset_id(
            "17B-197.sb34663512.eb34806505.58108.78427738426"
        )

        assert metadata == {
            "spl": "uid://evla/execblock/8fbfb54b-d141-42fe-b079-609339a69cba",
            "bands": "Ka",
            "array_config": "D",
            "obs_start_time": "1234567",
            "obs_end_time": "123598898",
        }
